use std::collections::BTreeMap;

use crate::util::index::{Dim, Ix2};

fn parse_nums(input: &str) -> BTreeMap<Dim<[i64; 2]>, u32> {
    let mut nums = BTreeMap::new();

    for (lnr, line) in input.lines().enumerate() {
        let mut line = line.chars().enumerate();
        while let Some((x, c)) = line.next() {
            if c.is_ascii_digit() {
                let mut num = (c as u8 - b'0') as u32;
                let mut end = x;
                for (idx, c) in line.by_ref() {
                    if c.is_ascii_digit() {
                        num = num * 10 + (c as u8 - b'0') as u32;
                        end = idx;
                    } else {
                        break;
                    }
                }
                for x2 in x..=end {
                    nums.insert(Dim([x2 as i64, lnr as i64]), num);
                }
            }
        }
    }
    nums
}
pub fn part1(input: &str) -> impl std::fmt::Display {
    let nums = parse_nums(input);

    input
        .lines()
        .enumerate()
        .flat_map(|(lnr, line)| {
            line.chars()
                .enumerate()
                .filter(|(_, c)| *c != '.' && !c.is_ascii_digit())
                .map(|(cnr, _)| {
                    let coord = Ix2::from([cnr as i64, lnr as i64]);
                    let mut vals = coord
                        .neighbors()
                        .iter()
                        .filter_map(|idx| nums.get(idx))
                        .collect::<Vec<_>>();
                    vals.dedup();
                    vals.into_iter().sum::<u32>()
                })
                .collect::<Vec<_>>()
        })
        .sum::<u32>()
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    let nums = parse_nums(input);

    input
        .lines()
        .enumerate()
        .flat_map(|(lnr, line)| {
            line.chars()
                .enumerate()
                .filter(|(_, c)| *c == '*')
                .filter_map(|(cnr, _)| {
                    let coord = Ix2::from([cnr as i64, lnr as i64]);
                    let mut vals = coord
                        .neighbors()
                        .iter()
                        .filter_map(|idx| nums.get(idx))
                        .collect::<Vec<_>>();
                    vals.dedup();
                    if vals.len() == 2 {
                        return Some(vals.into_iter().product::<u32>());
                    };
                    None
                })
                .collect::<Vec<_>>()
        })
        .sum::<u32>()
}

#[test]
fn test_part1() {
    assert_eq!("", part1("").to_string())
}

#[test]
fn test_part2() {
    assert_eq!("", part2("").to_string())
}
