use crate::util::parse::ParseExt;

pub fn part1(input: &str) -> impl std::fmt::Display {
    input
        .lines()
        .map(|line| {
            let (_, rest) = line.split_once(':').unwrap();
            let (winning, numbers) = rest.split_once('|').unwrap();
            let winning = winning.unsigned::<u32>().collect::<Vec<_>>();
            let numbers = numbers.unsigned::<u32>().filter(|n| winning.contains(n)).count() as u32;
            (numbers > 0).then(|| 2u32.pow(numbers - 1)).unwrap_or(0)
        })
        .sum::<u32>()
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    let lines = input.lines().collect::<Vec<_>>();
    let len = lines.len();
    let mut mults = vec![1; len];

    lines
        .iter()
        .enumerate()
        .map(|(i, line)| {
            let (_, rest) = line.split_once(':').unwrap();
            let (winning, numbers) = rest.split_once('|').unwrap();
            let winning = winning.unsigned::<u32>().collect::<Vec<_>>();
            let numbers = numbers.unsigned::<u32>().filter(|n| winning.contains(n)).count();
            let mul = mults[i];
            mults
                .iter_mut()
                .take((i + numbers + 1).min(len))
                .skip(i + 1)
                .for_each(|c| *c += mul);
            mul
        })
        .sum::<u32>()
}

#[test]
fn test_part1() {
    assert_eq!("", part1("").to_string())
}

#[test]
fn test_part2() {
    assert_eq!("", part2("").to_string())
}
