use crate::util::parse::ParseExt;

pub fn part1(input: &str) -> impl std::fmt::Display {
    let mut lines = input.lines();
    let times = lines.next().unwrap();
    let dists = lines.next().unwrap();

    times
        .signed::<i32>()
        .zip(dists.signed())
        .map(|(lim, dist)| {
            (0..dist)
                .filter_map(|d| (d * (lim - d) > dist).then_some(1))
                .sum::<u32>()
        })
        .product::<u32>()
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    let mut lines = input.lines();
    let time = lines
        .next()
        .unwrap()
        .chars()
        .filter(|a| a.is_ascii_digit())
        .collect::<String>().as_str()
        .parse_signed::<i128>();
    let dist = lines
        .next()
        .unwrap()
        .chars()
        .filter(|a| a.is_ascii_digit())
        .collect::<String>().as_str()
        .parse_signed::<i128>();

    (0..time)
        .filter_map(|d| (d * (time - d) > dist).then_some(1))
        .sum::<i128>()
}

#[test]
fn test_part1() {
    assert_eq!("", part1("").to_string())
}

#[test]
fn test_part2() {
    assert_eq!("", part2("").to_string())
}
